/*
 * example.c
 *
 * The Evil Fly Head Curse, a test example Palace server plugin.
 *
 * This plugin filters talk messages.
 *
 * If a user types "curse" (all by itself), the user acquires the Evil Fly Head
 * Curse. Typing "curse" again turns it off. It's a toggle.
 *
 * If a user types "curse you all", the same effect happens but the effect is
 * global--all users on the server acquire the curse. As with "curse", this is
 * also a toggle.
 *
 * The effects of these two toggles are independent of one another. You can
 * be effected by the individual curse, the global curse, or both.
 *
 * The global curse eventually wears off, but the individual curses stick
 * around until they are explicitly removed.
 *
 * What the Evil Fly Head Curse does is convert talk messages into FlySpeak.
 * FlySpeak is like English, except that the vocabulary is more limited. All
 * words consist of strings of the form [bB][zZ]*, e.g., "bzzz", "bz",
 * "bzzzzz", "Bzzz", "BzzZzz", etc.
 *
 * Copyright 1999 Communities.com. All rights reserved worldwide.
 */

#include "s-pluginapi.h"
#include <ctype.h>

/** Switch indicating the state of the global curse */
static Boolean GlobalFlyMode = FALSE;

static void flySpeak(char *text);
static Boolean pestControl(PluginID pluginID, char *text, ServerUserRec *user, Boolean *userFlyMode);

/**
 * Initialize the plugin.
 *
 * @param pluginID  This plugin's ID
 * @param argc  Count of args from the plugin config file (not used here)
 * @param argv  The plugin config file args themselves (also not used here)
 */
  AnyType initializeServerPlugin(PluginID pluginID, int argc, char *argv[])
{
    /* Each user gets a Boolean for the state of their own personal curse. */
    setPluginUserInfoSize(pluginID, sizeof(Boolean));

    /* Only plugin state needed is the plugin's ID, so we just return that. */
    return (AnyType) pluginID;
}

/**
 * Deal with a new user logon. Users begin life with their personal curses
 * turned off (lucky them!).
 *
 * @param state  The server state (not used here)
 * @param pluginDat  Our plugin state -- in our case it's the plugin ID
 * @param user  The new user
 */
  void
noteUserCreation(ServerState *state, AnyType pluginDat, ServerUserRec *user)
{
    Boolean *userFlyMode = (Boolean *) getPluginUserInfo((PluginID) pluginDat, user);
    *userFlyMode = FALSE;
}

/**
 * Deal with a timer event. We use timer events to enable the global curse to
 * wear off.
 *
 * @param state  The server state (not used here)
 * @param pluginDat  Our plugin state (also not used here)
 * @param timerDat  Timer-specific state (also not used; boy this is sure dull)
 */
  void
handleTimerEvent(ServerState *state, AnyType pluginDat, AnyType timerDat)
{
    if (GlobalFlyMode) {
        GlobalFlyMode = FALSE;
        GlobalMessage("The Evil Fly Head Curse epidemic has worn off");
    } else {
        GlobalMessage("The Evil Fly Head Curse epidemic would have worn off, but somebody already stopped it");
    }
}

/**
 * Process a message. Filter talk messages to look for the curse-toggling
 * commands and possibly flyify the users' speech.
 *
 * @param state  The server state
 * @param pluginDat  Our plugin state -- in our case it's the plugin ID
 * @param msg  The message we are filtering
 *
 * @return The filtered message
 */
  ClientMsg * handleMessage(ServerState *state, AnyType pluginDat, ClientMsg *msg)
{
    /* All we care about are talk messages */
    if (msg->eventType == MSG_TALK || msg->eventType == MSG_XTALK) {
        ClientMsg_xTalk *xmsg = (ClientMsg_xTalk *) &msg->msg;
        char buf[256];
        char *text;
        int textLength;
        Boolean *userFlyMode = (Boolean *) getPluginUserInfo((PluginID) pluginDat, state->currentUser);

        if (msg->eventType == MSG_TALK) {
            /* Normal talk messages just have plain text... */
            ClientMsg_talk *tmsg = (ClientMsg_talk *) &msg->msg;
            text = tmsg->text;
        } else {
            /* ...but xtalk messages need to be "decrypted". */
            if (msg->length < sizeof(uint16)) {
                /* Abort if badly formed xstring */
                return msg;
            }
            textLength = *(uint16 *)(xmsg->text) - 3;
            if (textLength > 255) {
                /* Avoid nasty buffer overrun opportunity */
                return msg;
            }
            DecryptCString((StringPtr) xmsg->text + 2, buf, textLength, FALSE);
            text = buf;
        }
        /* Handle curse toggling commands, if present */
        if (pestControl((PluginID) pluginDat, text, state->currentUser, userFlyMode)) {
            /* Consume commands */
            return NULL;
        }

        /* Invoke the curse, if applicable */
        if (GlobalFlyMode || *userFlyMode) {
            flySpeak(text);

            /* If we changed xtalk text, we need to re-encrypt */
            if (msg->eventType == MSG_XTALK) {
                EncryptCString(buf, (StringPtr) xmsg->text + 2, textLength);
            }
        }
    }
    return msg;
}

/**
 * Flyify a string.
 *
 * @param text  The text to be converted into FlySpeak. It's modified in place.
 */
  static void flySpeak(char *text)
{
    Boolean start = TRUE;
    while (*text) {
        if (islower(*text)) {
            *text = start ? 'b' : 'z';
            start = FALSE;
        } else if (isupper(*text)) {
            *text = start ? 'B' : 'Z';
            start = FALSE;
        } else if (isspace(*text) || (ispunct(*text) && *text != '\'')) {
            start = TRUE;
        }
        ++text;
    }
}

/**
 * Look for and process curse-toggling commands.
 *
 * @param text  The text to be examined
 * @param user  The user whose text this is
 * @param userFlyMode  Pointer to the user's individual curse control flag
 *
 * @return TRUE if a command was found and processed, FALSE otherwise
 */
  static Boolean pestControl(PluginID pluginID, char *text, ServerUserRec *user, Boolean *userFlyMode)
{
    if (strcmp(text, "curse you all") == 0) {
        /* Toggle the global curse flag and notify *everybody* */
        GlobalFlyMode = !GlobalFlyMode;
        if (GlobalFlyMode) {
            scheduleTimerEvent(pluginID, 0, 30); /* expire in 30 seconds */
            GlobalMessage("Everyone now has the Evil Fly Head Curse");
        } else {
            GlobalMessage("The Evil Fly Head Curse epidemic has ended");
        }
        return TRUE;
    } else if (strcmp(text, "curse") == 0) {
        /* Toggle the individual curse flag and notify the victim */
        *userFlyMode = !*userFlyMode;
        if (*userFlyMode) {
            UserPrivateMessage(user, "You now have the Evil Fly Head Curse");
        } else {
            UserPrivateMessage(user, "Your Evil Fly Head Curse is lifted");
        }
        return TRUE;
    }
    return FALSE;
}
